({
    // INFO => directory containing application (relative from this file)
    //The top level directory that contains your app. If it is not specified,
    //then baseUrl below is the anchor point for finding things. If specified,
    //all the files from the app directory will be copied to the dir:
    appDir: "../",

    // => INFO: directory path = PRODUCTION folder
    //The directory path to save the output. If not specified, then
    //the path will default to be a directory called "build" as a sibling
    //to the build file. All relative paths are relative to the build file.
    dir: "../dir",

    // INFO => enable whitespace compression on HTML templates pulled via text! (HACK)
    // https://github.com/tvavrys/text/commit/91edea9af4cbd6d6119b53a61dd56ecc4291105b
    config: {
      text: {
        removeWhitespace: true
      }
    },

    // INFO => mapping required for css! plugin (require-css/csso)
    // https://github.com/guybedford/require-is/blob/master/example/build.js
    map: {
      '*': {
        'css': '../js/plugins/require-css/css'
      }
    },

    // INFO => base url for all js files
    // INFO => If appDir is set, then baseUrl should relative to appDir.
    //By default, all modules are located relative to this path. If baseUrl
    //is not explicitly set, then all modules are loaded relative to
    //the directory that holds the build file. 
    baseUrl: "js",

    paths: {
        controller:   '../js/controller'
      // requirejs plugin paths
      , text:         '../js/text'
      , json:         '../js/plugins/requirejs-plugins/json'
      , async:        '../js/plugins/requirejs-plugins/async'
      , css:           '../js/plugins/require-css/css'
      , normalize:     '../js/plugins/require-css/normalize'
      //, font:         '../js/plugins/requirejs-plugins/font'
      //, propertyParser:'../js/plugins/requirejs-plugins/propertyParser'

      // main module
      , overrides:    '../js/overrides'
      , jquery:       '../js/libs/jquery/jquery'
      , jqm:          '../js/libs/jqm/jqm'
      , i18next:      '../js/plugins/i18next/i18next'
      , renderjs:     '../js/plugins/renderjs/renderjs'
      , localstorage: '../js/libs/jio/localstorage'
      , queries:      '../js/libs/jio/complex_queries'
      , md5:          '../js/libs/jio/md5'
      , jio:          '../js/libs/jio/jio'

      // page modules
      , index:         '../modules/index/index'
      , legal:         '../modules/legal/legal'
      // modules
      , gmaps:         '../js/plugins/jquery-ui-map/jquery.ui.map.module'
      , plot:          '../js/plugins/jqplot/jqplot.module'
      , header:        '../modules/header/header'
      , translate:     '../modules/translate/translate'
      , social:        '../modules/social/social'
      , navigation:    '../modules/navigation/navigation'
      , configure:     '../modules/configure/configure'
      , results:       '../modules/results/results'
      , about:         '../modules/about/about/about'
      , thanks:        '../modules/thanks/thanks'
      , mailto:        '../modules/mailto/mailto'
      , mapper:        '../modules/mapper/mapper'
    },

    // => INFO shims (same as in main.js)
    //If shim config is used in the app during runtime, duplicate the config
    //here. Necessary if shim config is used, so that the shim's dependencies
    //are included in the build. Using "mainConfigFile" is a better way to
    //pass this information though, so that it is only listed in one place.
    //However, if mainConfigFile is not an option, the shim config can be
    //inlined in the build config.
    shim: {
        'overrides':     { deps: ['jquery'] }
      , 'jqm':           { deps: ['jquery'], exports: 'mobile' }
      , 'renderjs':      { deps: ['jquery', 'jio', 'md5', 'localstorage', 'queries'], exports: 'RenderJs' }
      , 'jio':           { deps: ['md5'] }
      , 'localstorage':  { deps: ['jio'] }
      , 'queries':       { deps: ['jio'] }
      , 'maps':          { deps: ['jquery', 'jqm']}
    },

 
    keepBuildDir: true,

    optimize: "uglify2",

    //files.
    skipDirOptimize: false,

    //source maps as ".js.src" files.
    generateSourceMaps: false,

    //files.
    normalizeDirDefines: "skip",

    //returns. (r.js 1.0.8+)
    optimizeCss: "standard",

    inlineText: true,

    //Allow "use strict"; be included in the RequireJS files.
    //Default is false because there are not many browsers that can properly
    //process and give errors on code for ES5 strict mode,
    //and there is a lot of legacy code that will not work in strict mode.
    useStrict: false,

    skipModuleInsertion: false,

    optimizeAllPluginResources: false,

    //Finds require() dependencies inside a require() or define call. By default
    //this value is false, because those resources should be considered dynamic/runtime
    //calls. However, for some optimization scenarios,
    //Introduced in 1.0.3. Previous versions incorrectly found the nested calls
    //by default.
    findNestedDependencies: true,

    //If set to true, any files that were combined into a build layer will be
    //removed from the output folder.
    removeCombined: true,

    //List the modules that will be optimized. All their immediate and deep
    //dependencies will be included in the module's file when the build is
    //done. If that module or any of its dependencies includes i18n bundles,
    //only the root bundles will be included unless the locale: section is set above.
    modules: [
      {
          name: "main"
        , separateCSS: true
        , excludeShallow: ["gmaps, plot"]
      },
      {
          name: "gmaps"
        , exclude: ["jquery", "jqm"]
      },
      {
          name: "plot"
        , include: [
              "../js/plugins/jqplot/plugins/jqplot.barRenderer"
            , "../js/plugins/jqplot/plugins/jqplot.logAxisRenderer"
            , "../js/plugins/jqplot/plugins/jqplot.categoryAxisRenderer"
            , "../js/plugins/jqplot/plugins/jqplot.canvasAxisTickRenderer"
            , "../js/plugins/jqplot/plugins/jqplot.canvasTextRenderer"
            , "../js/plugins/jqplot/plugins/jqplot.pointLabels"
            , "../js/plugins/jqplot/plugins/jqplot.enhancedLegendRenderer"
        ]
        , exclude: ["jquery", "normalize", "css"]
      },
      // ================  page modules ================ 
      {
          name: "index",
          exclude: ["text", "normalize", "css"],
          // this will make the modules part of the build
          // header, social, navigation ~ gzip <2k, so include
          // otherwise request once and then load from cache
          // to declare a module "common" with all gadgets/modules
          // which can found on every page and request this once
          // so it's cached
          include: ["configure", "header", "social", "navigation"]
      },
      {
          name: "legal",
          exclude: ["text", "normalize", "css"],
      },
      // ================ content modules  ================ 
      {
          name: "header",
          exclude: ["text", "normalize", "css"],
          include: ["translate"]
      },
      {
          name: "translate",
          exclude: ["text", "normalize", "css"]
      },
      {
          name: "configure",
          exclude: ["text", "normalize", "css", "json"]
      },
       {
          name: "social",
          exclude: ["text", "normalize", "css"]
      },
      {
          name: "navigation",
          exclude: ["text", "normalize", "css"]
      },
      {
          name: "mapper",
          exclude: ["text", "normalize", "css", "json"],
          include: ["gmaps"]
      }

       ],

    fileExclusionRegExp: /^(dir|py|build|inputs|.*~|.git|.gitignore)$/,

    //By default, comments that have a license in them are preserved in the
    //output. However, for a larger built files there could be a lot of
    //comment files that may be better served by having a smaller comment
    //at the top of the file that points to the list of all the licenses.
    //This option will turn off the auto-preservation, but you will need
    //work out how best to surface the license information.
    preserveLicenseComments: true,

    //Sets the logging level. It is a number. If you want "silent" running,
    //set logLevel to 4. From the logger.js file:
    //TRACE: 0,
    //INFO: 1,
    //WARN: 2,
    //ERROR: 3,
    //SILENT: 4
    //Default is 0.
    logLevel: 0,

    //Introduced in 2.1.3: Some situations do not throw and stop the optimizer
    //when an error occurs. However, you may want to have the optimizer stop
    //on certain kinds of errors and you can configure those situations via
    //this option
    throwWhen: {
        //If there is an error calling the minifier for some JavaScript,
        //instead of just skipping that file throw an error.
        optimize: true
    },

    waitSeconds: 7

})