define([
    'text!../modules/results/results.html!strip'
  , 'css!../modules/results/results'
  ],
  function (source) {
    var response = {};
    response.data = source;
    response.callback = function () {
      var timer = null,
      stored_regex = {
        // TODO: waste of RegExp, albeit... used on every table upload, so better cache here
        re_href: new RegExp("tmp_href", "g"),             // link to provider
        re_title : new RegExp("tmp_title", "g"),          // text info for link to provider
        re_img_path : new RegExp("tmp_image_path", "g"),  // path to logo of provider
        re_name : new RegExp("tmp_name", "g"),            // name of provider
        re_data_loss : new RegExp("tmp_data_loss", "g"),  // calculated data loss
        re_percentile : new RegExp("tmp_percentile", "g"),// calculated uptime
        re_price: new RegExp("tmp_price", "g"),           // provider cost
        re_alias: new RegExp("tmp_alias", "g"),           // machine alias
        re_count : new RegExp("tmp_count", "g"),          // loop counter for popup
        re_comp: new RegExp("tmp_comp", "g"),             // comp name of machine (reference)
        re_currency: new RegExp("tmp_currency", "g"),     // currency of quotation (hardcoded $)
        re_py: new RegExp("tmp_pystone", "g"),            // lookup pystone average
        re_sql: new RegExp("tmp_sqlbench", "g"),          // lookup sqlbench average
        re_unit_py: new RegExp("tmp_unit_py", "g"),       // performance units pystone
        re_unit_sql: new RegExp("tmp_unit_sql", "g")      // performance units sqlbench 
      },

      getParameterByName = function (name) {
        return decodeURIComponent(
          (RegExp(name + '=' + '(.+?)(&|$)').exec($.fn.fauna.formdata)||[,""])[1]
        );
      },

      getObjectSize = function (obj) {
        var size = 0, key;
        for (key in obj) {
          if (obj.hasOwnProperty(key)) {
            size += 1;
          }
        }
        return size;
      },

      arrayPosition = function (element, array) {
        var i, l = array.length;
        for (i = 0; i < l; i += 1) {
          if (array[i] === element) {
            return i;
          }
        }
        return null;
      },

      arrayMax = function( array ){
        return Math.max.apply( Math, array );
      },

      arrayMin = function( array ){
        return Math.min.apply( Math, array );
      },

      roundNumber = function (number) {
        var digits = 2,
        multiple = Math.pow(10, digits);
        rndedNum = Math.round(number * multiple) / multiple;
        return rndedNum;
      },

      makeColors = function (array, get, axis) {
        var pos, colorObject = {}, colorArray = [];
        //if (get === "max") {
        //  pos = arrayMax(array);
        //} else {
        //  pos = arrayMin(array);
        //}
        // highlight min/max value, rest gray... not working well with 4 series
        //for (i = 0; i < array.length; i += 1) {
        //  if (array[i] === pos || array.length === 1) {
        //    colorArray.push("#2DBCF8");
        //  } else {
        //    colorArray.push("#ccc");
        //  }
        //}
        //colorObject.series_colors = colorArray;
        //add rest, too...
        if (axis === "log") {
          colorObject.yaxis = "yaxis";
        } else {
          colorObject.yaxis = "y2axis";
        }
        colorObject.pointLabels = {edgeTolerance: -50}
        // colorObject.legend = "#ccc";
        return colorObject;
      },

      getTemplate = function (block, context) {
        var regex = RegExp('<!-- ' + block + '(.)+' + block + '(.)+?-->'),
            tmpl = context.match(regex);
        return tmpl.length ? tmpl[0].replace(/<!--[\s\S]*?-->/g, '') : '';
      },

      tableRender = function (table) {

        table
          // refresh
          .filter(function() { return $(this).jqmData('bound') === true; })
          .table('refresh')
          .end()
          // create table on first run
          .filter(function() { return $(this).jqmData('bound') !== true; })
          .jqmData('bound', true)
          .attr('data-role', 'table')
          .table({ create: function( event, ui ) {} })
          .table({ tablecreate: function( event, ui ) {} });
      },

      // add information to details popup
      renderDetails = function (err, response, reference) {
        // add a graph, so pull all files needed from jqPlot here
        if (response) {
          require(['plot'],
            function (detail_plot) {
              var plotOptions,
                features = $('.perf_table .perf_feature'),
                response_values = [],
                i,
                response_machine,
                test,
                key,
                value,
                test_object,
                test_score,
                data_series_sql = [],
                data_series_py = [],
                data_series_sql_ticks = [],
                data_series_py_ticks = [],
                plotPy,
                plotSQL,
                collapsibleSets = $('#collapsible_python, #collapsible_sql');

              // set machine configuration
              $.fn.fauna.jio_item_config.get(
                {"_id" : response._id},
                function(err, item_response){
                  var last_val;
                  $('span.machine_features_name').text(item_response._id);
                  $('span.machine_features_provider').text(item_response.provider);
                  $('span.machine_features_cpu').text(item_response.cpu_frequency_i);
                  $('span.machine_features_cpu_type').text(item_response.cpu_type_i);
                  $('span.machine_features_memory').text(item_response.memory_size_i);
                  $('span.machine_features_harddisk').text(item_response.storage_capacity_i);

                  for (test in response) {
                    if (response.hasOwnProperty(test)) {
                      test_object = response[test];
                      if (typeof test_object === "object") {
                        for (key in test_object) {
                          if (test_object.hasOwnProperty(key)) {
                            test_score = test_object[key];
                            // plot series
                            if (test === "pystone" && key === "score") {
                              last_val = 0;
                              for (parameter in test_score) {
                                value = test_score[parameter];
                                if (test_score.hasOwnProperty(parameter)) {
                                  data_series_py.push([value-last_val]);
                                  last_val = value;
                                  data_series_py_ticks.unshift(parameter);
                                }
                              }
                            } else if (test === "sqlbench" && key === "time") {
                              last_val = 0;
                              for (parameter in test_score) {
                                value = test_score[parameter];
                                if (test_score.hasOwnProperty(parameter)) {
                                  data_series_sql.push([value-last_val]);
                                  last_val = value;
                                  data_series_sql_ticks.unshift(parameter);
                                }
                              }
                            }
                            // default arrays
                            for (parameter in test_score) {
                              if (test_score.hasOwnProperty(parameter)) {
                                value = test_score[parameter];
                                response_values.unshift(value);
                                }
                            }
                          }
                        }
                      }
                    }
                  }

                  // fill listviews - TODO: build dynamically or make from
                  // plot series.
                  for (i = 0; i < features.length; i += 1) {
                    features[i].innerHTML = parseFloat(response_values[i]).toFixed(2);
                  }

                  $.jqplot.config.enablePlugins = true;

                  // plot
                  // we could plot against all visible records?
                  plotOptions = {
                    stackSeries: true,
                    seriesDefaults:{
                      renderer:$.jqplot.BarRenderer,
                      rendererOptions: {
                          // Put a 30 pixel margin between bars.
                          barMargin: 30,
                          barWidth: 50
                      },
                      pointLabels: {
                        show: true,
                        stackedValue: true,
                        location: 'e',
                        edgeTolerance: -15,
                        xpadding: 40
                      }
                    },
                    axes: {
                      xaxis: {
                          renderer: $.jqplot.CategoryAxisRenderer,
                          label: item_response.provider,
                          ticks: ['']
                      },
                      yaxis: {
                        // Don't pad out the bottom of the data range.  By default,
                        // axes scaled as if data extended 10% above and below the
                        // actual range to prevent data points right on grid boundaries.
                        // Don't want to do that here.
                        renderer: $.jqplot.LinearAxisRenderer,
                        padMin: 0
                      }
                    },
                    highlighter: { show: false }
                  };

                    collapsibleSets.each(function () {
                      var tag = this.getAttribute('id').split("collapsible_")[1],
                        plot_id,
                        data_series;

                      if (tag === "python") {
                        plot_id = 'chart_py';
                        data_series = data_series_py;
                      } else if (tag === "sql") {
                        plot_id = 'chart_sql';
                        data_series = data_series_sql;
                      }
                      // plotOptions.axes.yaxis.min = data_series[2][0];
                      // plotOptions.axes.yaxis.max = data_series[2][0]*2;
                      // document...
                      document.getElementById(plot_id).innerHTML = '';

                      if ($(this).hasClass("ui-collapsible-collapsed") === true) {
                        // JQM bug, not possible to bind to collapsibleexpand
                        $(this).find('.ui-collapsible-heading')
                          .on('click.jqplot', function () {
                            $.jqplot(plot_id, data_series, plotOptions);
                            $(this).off('jqplot');
                          });
                      } else {
                        $.jqplot(plot_id, data_series, plotOptions)
                      }
                    }
                  );
                }
              );
            }
          );
        } else {
          // TODO: something else here
          // $('.all_popups').popup( "close" )
        }
      },

      getDetails = function (reference) {
        // first call
        if ($.fn.fauna.jio_performance_details === undefined) {
          // create performance_details jIO
          // if more than 6 records are used.... think about an alternative
          // otherwise this will also stall for a few seconds once reaching
          // a few 100s records
          $.fn.fauna.jio_performance_details = jIO.newJio({
            "type": "local",
            "username": "performance_details",
            "application_name": "cloudquote"
          });

          // pull data and store in JIO
          ajaxObject = {
            url: "modules/results/machine_performance.json",
            //url: "http://www.erp5.com/TestResultModule_getCloudPerformanceStatisticAsJson",
            type: "GET",
            target: "",
            successHandler : function (data) {
              var machine_performance, m, machine_id;

              // look up table needs faster access
              $.fn.fauna.performance_avgs = {}

              // loop data
              for (machine in data) {
                if (data.hasOwnProperty(machine)) {
                  m = data[machine]
                  machine_id = machine.split("-")[2].replace("COMP", "COMP-");
                  machine_performance = {
                    _id: machine_id,
                    pystone: {
                      time: {
                        min:m["pystone"]["time"]["min"],
                        avg:m["pystone"]["time"]["avg"],
                        max:m["pystone"]["time"]["max"]
                      },
                      score: {
                        min:m["pystone"]["score"]["min"],
                        avg:m["pystone"]["score"]["avg"],
                        max:m["pystone"]["score"]["max"]
                      }
                    },
                    sqlbench: {
                      time: {
                        min:m["sqlbench"]["time"]["min"],
                        avg:m["sqlbench"]["time"]["avg"],
                        max:m["sqlbench"]["time"]["max"]
                      }
                    }
                  };
                  // table will show pystone score and sqlbench time!
                  $.fn.fauna.performance_avgs[machine_id] = [m["pystone"]["score"]["avg"],m["sqlbench"]["time"]["avg"]];
                  $.fn.fauna.jio_performance_details.put(machine_performance);
                }
              };
              if (reference !== undefined) {
                $.fn.fauna.jio_performance_details.get({"_id":reference},
                  function (err, response) {
                    renderDetails(err, response, reference);
                  }
                );
              }
            },
            errorHandler : function (XMLHttpRequest, textStatus, errorThrown) {
              alert("error trying to retrieve machine details, please refresh page");
            }
          };
          $.fn.util.ajaxContent(ajaxObject);
        } else {
          if (reference !== undefined) {
            $.fn.fauna.jio_performance_details.get({"_id":reference},
              function (err, response) {
                renderDetails(err, response, reference);
              }
            );
          }
        }
      },

      renderData = function (create, field, direction, search_term) {
        var i, j = 0, param, references, query_string = "", sort_string = "",
          criteria_string = "", sort_by = field ? field : 'percentile',
          sort_direction = direction ? direction : 'descending',
          provider_filter = $('#form_provider_view').val(),
          query_object = {
            "query":{
              "filter": {
                  "limit":[0,10],
                  "sort_on":[[sort_by,sort_direction]],
                  "select_list":[]
              },
              "wildcard_character":'%'
            }
          };

        // TODO: simplify!!!
        // ================== Complex Query Builder =================
        if ($.fn.fauna.formdata !== "" || provider_filter === "all") {

          // construct search term from selected criteria
          references = getObjectSize($.fn.fauna.references);
          for (i in $.fn.fauna.references) {
            if ($.fn.fauna.references.hasOwnProperty(i)) {
              param = getParameterByName(i);
              if (param !== "" && param !== "default") {
                query_string += i + ": = " + param + " AND ";
                criteria_string += i + ": " + param + " | ";
              }
              if (references-1 === j) {
                query_string = query_string.substring(0, query_string.length - 5);
                criteria_string = criteria_string.substring(0, criteria_string.length -3);
                $('.search_criteria').text(criteria_string);
              }
            }
            j += 1;
          }
          if (search_term && provider_filter !== "all") {
            query_string += " AND (";
          }
        } else {
          query_string = "";
          $('.search_criteria').text();
        }

        // filter - look up all fields and custom-made searchtext
        if (search_term) {
          sort_string = 
            "percentile: = %" + search_term +
            "% OR downtime: = %" + search_term +
            "% OR totaltime: = %" +  search_term +
            "% OR _id: = %" + search_term +
            "% OR provider: = %" + search_term +
            "% OR searchtext: = %" + search_term +
            "% OR alias: = %" + search_term + "%";
          if (query_string !== "" && provider_filter !== "all") {
            sort_string += ")";
          }
        } else {
          // we might be switching view from a provider filter_value
          // if so, don't load full result set, only selected provider
          if(provider_filter !== "default" && 
            provider_filter !== null && 
              provider_filter !== "all") {
            sort_string = "provider: = %" + provider_filter + "%";
            if (query_string !== "") {
              sort_string += ")";
            }
          }
        }

        // concat all search strings
        if ($.fn.fauna.formdata !== "" || sort_string !== "" || query_string !== "") {
          query_object.query.query = query_string + sort_string;
          // deeplink safety (if deeplinking to results, clean the query string)
          if (query_object.query.query === "") {
            delete query_object.query.query;
          }
        }
        // ================== /QUERY DONE/ =================

        $.fn.util.spin('show');

        // ================== Retrieve Data =================
        $.fn.fauna.jio_item_config.allDocs(
          query_object,
          function (err, response){
            var table_target = $(".rendered_result_wrapper"), plot1;
            // switch between table and graph
            // make a table
            if ($("#form_results_view").val() === "display_list") {

              // show header in case hidden
              table_target.closest("table").find("thead").show();

              // get row-template
              require(['text!../tmp/tmp_results.html'], function (template_library) {
                var use_template = 'tmp_result_row',
                  allRows = '',
                  i,
                  j,
                  entry,
                  instance,
                  machine_alias,
                  machine_name,
                  machine_img,
                  machine_price,
                  machine_py,
                  machine_sql,
                  machine_currency,
                  machine_performance_unit = "s",
                  opts = "",
                  t = getTemplate(use_template, template_library);

              // add named providers to search-select
              if (create) {
                for (j = 0; j < $.fn.fauna.named_providers.length; j += 1) {
                  select_entry = $.fn.fauna.named_providers[j];
                  select_entry_lc = select_entry.toLowerCase();
                  opts += '<option value="'+select_entry_lc+'">'+select_entry+'</option>'
                }
                $('#form_provider_view').append(opts);
              }

              // set values for table row
              for (i = 0; i < response.length; i += 1) {
                entry = response[i];
                machine_currency = "$";
                machine_py_unit = " p/s";
                machine_sql_unit = " s";
                machine_price = $.fn.fauna.pricing[entry["_id"]]

                if (entry["provider"] !== "") {
                  machine_alias = entry["provider"];
                  machine_img = "img/providers/" + entry["provider"].toLowerCase() + ".png";
                } else {
                  machine_alias = entry["_id"];
                  machine_img = "";
                }
                if (entry["provider"] !== "") {
                  machine_name = entry["provider"] + " (" + entry["_id"] + ")";
                } else {
                  machine_name = entry["_id"];
                }
                if ($.fn.fauna.performance_avgs[entry["_id"]] === undefined) {
                  machine_py = "n/a";
                  machine_sql = "n/a";
                  machine_performance_unit = "";
                  machine_py_unit = "";
                  machine_sql_unit = "";
                } else {
                  machine_py = $.fn.fauna.performance_avgs[entry["_id"]][0].toFixed(0);
                  machine_sql = $.fn.fauna.performance_avgs[entry["_id"]][1].toFixed(0);
                }
                if ($.fn.fauna.pricing[entry["_id"]] === 0 ||
                  $.fn.fauna.pricing[entry["_id"]] === undefined) {
                  machine_price = "n/a"
                }
                if (machine_price === "n/a") {
                  machine_currency = "";
                }

                // create table row from template
                instance = t.replace('id="tmp_result_row"','')
                .replace(stored_regex.re_name, machine_alias)
                .replace(stored_regex.re_img_path, machine_img)
                .replace(stored_regex.re_percentile , roundNumber(entry["percentile"]))
                .replace(stored_regex.re_comp, entry["_id"])
                .replace(stored_regex.re_title, machine_alias)
                .replace(stored_regex.re_alias, machine_name)
                .replace(stored_regex.re_price, machine_price)
                .replace(stored_regex.re_currency, machine_currency)
                .replace(stored_regex.re_py, machine_py)
                .replace(stored_regex.re_sql, machine_sql)
                .replace(stored_regex.re_unit_py, machine_py_unit)
                .replace(stored_regex.re_unit_sql, machine_sql_unit);
                allRows += instance;

                machine_name = "";
                machine_alias = "";
              }

              // insert
              table_target
                .empty()
                .append(allRows)
                // enhance elements in the table rows
                .trigger("create");

              tableRender(table_target.parent('table'));
            });
          } else {
            // make a graph

            // clear the table and hide thead
            table_target.empty().closest("table").find("thead").hide();

            require(['plot'],
              function (table_plot) {

                // assigning plugins to $.jqplot takes too long, so
                // requiresJS runs before plugins are available...
                if ($.jqplot.BarRenderer === undefined) {
                  setTimeout(function(){
                    renderData(true);
                  },100);
                  return;
                }

                  var entry,
                    series_providers = [],
                    series_availability = [],
                    series_cost = [],
                    series_colors = [],
                    series_py = [],
                    series_sql = [],
                    useBarWidth,
                    config_xaxis,
                    plotOpts,
                    plotSeries = [],
                    plotLables = [],
                    // TODO: make this generic
                    toggle_inputs = $("#results_table-popup input"),
                    toggle_label,
                    i,
                    j

                  if (response.length !== 0) {
                    // create data series arrays from available info
                    for (i = 0; i < response.length; i += 1) {
                      entry = response[i];
                      if (entry["provider"] !== "") {
                        series_providers.push(entry["provider"]);

                        series_availability.push( roundNumber(entry["percentile"]/100) );
                        series_cost.push( $.fn.fauna.pricing[entry["_id"]] || entry["cost"] || 0 );

                        // we could jio.get, but it takes to long to create an array
                        // looping over matching jIO records. Only use the performance
                        // jIO for single lookups.
                        if ($.fn.fauna.performance_avgs[entry["_id"]] !== undefined) {
                          series_py.push( $.fn.fauna.performance_avgs[entry["_id"]][0]);
                          series_sql.push( $.fn.fauna.performance_avgs[entry["_id"]][1]);
                        } else {
                          series_py.push(0);
                          series_sql.push(0);
                        }
                      }
                    }

                    // override barWidth for single results
                    if (series_availability.length === 1) {
                      useBarWidth = 50;
                    }

                    $.jqplot.config.enablePlugins = true;

                    // x-axis
                    config_xaxis = {
                        renderer: $.jqplot.CategoryAxisRenderer,
                        label:'Providers',
                        ticks: series_providers,
                    }

                    // small screens
                    if ($.fn.fauna.screensize === "small") {
                      config_xaxis.tickRenderer = $.jqplot.CanvasAxisTickRenderer;
                      config_xaxis.tickOptions = { angle: -90 }
                    }

                    // add wrapper
                    table_target
                      .parent('table')
                      .find('thead tr td, thead tr th')
                      .attr('data-priority','persist')
                      .end()
                      .find('tbody')
                      .empty()
                      .append( $('<tr><td colspan="7"><div id="graph_container_cell"></div></td></tr>') );

                    plotOpts = {
                      title: 'Performance vs Uptime, Cost',
                      seriesDefaults:{
                          renderer:$.jqplot.BarRenderer,
                          rendererOptions:{ /* varyBarColor : true,*/ barWidth: useBarWidth }
                      },
                      tickDefaults: {
                          syncTicks: true,
                          useSeriesColor: true, 
                          autoscale: true,
                          alignTicks: true,
                          forceTickAt0: true
                      },
                      axes: {
                        xaxis: config_xaxis,
                        yaxis:{
                          label:'',
                          renderer:$.jqplot.LogAxisRenderer,
                        },
                        y2axis:{
                          label: '',
                          tickOptions: {
                            formatString: '%.3f '
                          }
                        }
                      },
                      highlighter: { show: false },
                      legend:{
                        renderer: $.jqplot.EnhancedLegendRenderer,
                        placement: 'outsideGrid',
                        location:'s',
                        show: true,
                        rendererOptions: {
                          numberRows: 1
                        }
                      }
                    };

                    // y-axis - only add series, if columntoggle is checked
                    // needs to be set after defining object to update accordingly
                    // TODO: this is not generic.. redo...
                    for (i = 0; i < toggle_inputs.length; i += 1) {
                      if (toggle_inputs[i].checked) {
                        switch (i) {
                          case 2:
                            // 0-1
                            plotSeries.push(series_availability);
                            plotLables.push("Availability (%)");
                            series_colors.push(makeColors(series_availability, "max", "norm"));
                            break;
                          case 3:
                            // 0-5
                            plotSeries.push(series_py);
                            plotLables.push("PyStone (p/s)");
                            series_colors.push(makeColors(series_py, "min", "log"));
                            break;
                          case 4:
                            // 0-700
                            plotSeries.push(series_sql);
                            plotLables.push("SQLBench (s)");
                            series_colors.push(makeColors(series_sql, "min", "log"));
                            break;
                          case 5:
                            // 0-1
                            plotSeries.push(series_cost);
                            plotLables.push("Cost ($/h)");
                            series_colors.push(makeColors(series_cost, "min", "norm"));
                            break;
                        }
                      }
                    }
                    plotOpts.legend.labels = plotLables;
                    plotOpts.series = series_colors;

                    // now we plot - only if a series is shown
                    if (plotSeries.length !== 0) {
                      plot1 = $.jqplot('graph_container_cell', plotSeries, plotOpts);

                      // add i18n attributes
                      $('.jqplot-title')
                        .addClass('t').attr('i18n','tx.gen.sys_vs_cost');
                      $('.jqplot-xaxis-label')
                        .addClass('t').attr('i18n', 'tx.gen.provider');
                      $('.jqplot-table-legend').eq(0).add('.jqplot-yaxis-label')
                        .addClass('t').attr('i18n', 'tx.gen.availability');
                      $('.jqplot-table-legend').eq(1).add('.jqplot-y2axis-label')
                        .addClass('t').attr('i18n', 'tx.gen.price');

                      tableRender(table_target.parent('table'));

                      // how to bind to resize for orientationchange/resize from a gadget?
                      // if I can't access gadget
                      $(window)
                        .filter(function() { return $(this).jqmData('plot_bound') !== true; })
                        .jqmData('plot_bound', true)
                        .on('resize orientationchange', function () {
                          // need this to autoAdjust cached bar-width:
                          $.each(plot1.series, function(index, series) {
                            series.barWidth = undefined;
                          });
                          // reset target, because it will be set to the previous allDocs result object
                          // which has been removed but is still used as a reference in jqPlot
                          if (plot1.target.closest('div[data-gadget]').length === 0) {
                            plot1.target = $('#graph_container_cell');
                          }
                          plot1.replot( plotOpts, { resetAxes: true });
                        });
                    }
                  }

              }
            );
          }

          // clear the formdata for the next queries
          $.fn.fauna.formdata = ""
          $.fn.util.spin('hide');
        });
      };

      // if item_config is undefined, changePage to index
      if ($.fn.fauna.jio_item_config === undefined) {
        window.location.href = "index.html";
      }

      // initialize
      renderData(true);
      getDetails();


      $(document).ready(function() {
        var gadget = RenderJs.getSelfGadget(),
          table_popup_inputs = "#results_table" + "-popup input";

        // fallback for old devices which cannot load <style> css
        if (gadget.dom.find(".el").css('width') !== "1px") {
          require(['text!../css/results.css'], function (t) {
            var x = '<style>'+t+'</style>';
            // why does this not updat the DOM, too?
            gadget.dom.append(x);
          });
        }

        // filter value storage (needed for timer)
        gadget.filter_value = $('.ui-input-search input').val();
        gadget.current_selection = "";

        // interaction listener
        gadget.translate = function(){
          $.fn.util.lang(gadget.dom);
        };

        // trigger JQM enhancement
        $(this)
          .trigger("render_enhance", {gadget: gadget.dom, pops:true})
          .on('change', table_popup_inputs, function () {
            // toggle graph columns only if we are displaying a graph
            if ($("#form_results_view").val() !== "display_list") {
              renderData();
            }
					});results.html

        // sorting
        gadget.dom
          .find('#form_results_sort')
          .filter(function() { return $(this).jqmData('bound') !== true; })
          .jqmData('bound', true)
          .on('change', function (e) {
            var field, direction;
            switch($(this).val()) {
              case "sort_provider_az":
                field = "_id";
                direction = "descending";
                break
              case "sort_provider_za":
                field = "_id";
                direction = "ascending";
                break
              case "sort_price_highest":
                field = "price";
                direction = "descending";
                break
              case "sort_price_lowest":
                field = "price";
                direction = "ascending";
                break
        //               case "sort_py_lowest":
        //                 field = "price";
      //                 direction = "ascending";
      //                 break
      //               case "sort_py_highest":
      //                 field = "price";
      //                 direction = "descending";
      //                 break
      //               case "sort_sql_lowest":
      //                 field = "price";
      //                 direction = "ascending";
      //                 break
      //               case "sort_sql_highest":
      //                 field = "price";
      //                 direction = "descending";
      //                 break
              case "sort_score_highest":
                field = "percentile";
                direction = "descending";
                break
              case "sort_score_lowest":
                field = "percentile";
                direction = "ascending";
                break
            }
            gadget.sort_field = field;
            gadget.sort_direction = direction;
            renderData(false, field, direction, null);
          });

        gadget.dom
        .on('tablebeforefilter.results_filter', '#results_table', function ( e, data ) {
          var inp = $( e.target ).parent().find('.ui-input-search input'),
            val = inp.val();
          // 1sec delay to allow entering multiple characters
          if (timer) {
            window.clearTimeout(timer);
          }
          if(val !== gadget.filter_value) {
            timer = window.setTimeout(function(){
              timer = null;
              gadget.filter_value = val;
              renderData(false, gadget.sort_field, gadget.sort_direction, val);
            },500);
          }
        })
        .on('change.results_filter', '#form_provider_view', function () {
          var val = $(this).val() === "all" ? "" : $(this).val();
          renderData(false, gadget.sort_field, gadget.sort_direction, val);
        })
        .on('change.display_mode', '#form_results_view', function () {
          renderData(false, gadget.sort_field, gadget.sort_direction, null);
        })
        .on('click.details_popup', '.popup_trigger', function () {
          var ref = $(this).attr("data-reference");
          getDetails(ref);
        })
        // HACK: should not do that
        .on('click.start_over', '.start_over', function () {
         $.fn.fauna.reset = true;
        })
        // unbind on hide
        .off('pagehide','.results_filter .details_popup .display_mode');
      });
    };
    return response;
  }
);
